import React, { Component, PropTypes } from 'react';
import { GoogleMapLoader, GoogleMap, Marker } from 'react-google-maps';
import debounce from 'lodash.debounce';
import sortby from 'lodash.sortby';

import TextField    from '../TextField';
import Button       from '../Button';
import Autocomplete from '../Autocomplete';
import { areas }    from './kwAreas';

import './AddressBox.scss';

const areasOptions = sortby(areas.map(it => ({
  ...it,
  groupId: it.governorate,
  label  : it.englishName,
  value  : it.englishName
})), [ 'groupId', 'label' ]);

class AddressBox extends Component {

  static propTypes = {
    showMap: PropTypes.bool,
    value  : PropTypes.shape({
      location       : PropTypes.shape({
        latitude : PropTypes.number,
        longitude: PropTypes.number
      }),
      block          : PropTypes.string,
      street         : PropTypes.string,
      apartment      : PropTypes.string,
      floor          : PropTypes.string,
      instructions   : PropTypes.string,
      buildingNumber : PropTypes.string,
      area           : PropTypes.string,
      avenue         : PropTypes.string,
      draggableMarker: PropTypes.bool
    })
  };

  static defaultProps = {
    showMap        : false,
    draggableMarker: true,
    value          : { location: {} },
  };

  constructor(props) {
    super(props);

    this.state = {
      open : false,
      zoom : 0,
      value: { ...(props.value || {}) }
    };

    this.handleFocus    = this.handleFocus.bind(this);
    this.geocodeAddress = debounce(this.geocodeAddress, 1000);
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevProps.value != this.props.value) {
      this.setState({ value: this.props.value });

      if (this.state.zoom == 0 && this.props.value.location)
        this.setState({ zoom: 15 });
    }

    if (!prevState.open && this.state.open) {
      this.refs.details.querySelector('input').focus();

      const closeListener = (e) => {
        var elem = e.target;

        while (elem.parentNode != null) {
          if (elem == this.refs.details) return;
          elem = elem.parentNode;
        }

        e.stopImmediatePropagation();
        window.removeEventListener('click', closeListener);
        this.setState({ open: false });
      };

      window.addEventListener('click', closeListener);
    }

    if (prevState.open && !this.state.open)
      this.props.onBlur && this.props.onBlur();
  }

  sortBlocks(blocks) {
    return sortby(blocks.map(it => {
      var block      = Object.keys(it)[ 0 ];
      var numberPart = +block.replace(/\D/g, '') || 0;
      return {
        value     : block,
        label     : block,
        numberPart: numberPart
      };
    }), [ 'numberPart', 'value' ])
  }

  getFieldProps(fieldName) {
    return {
      value   : this.state.value[ fieldName ],
      onChange: (e) => {

        let stateChanges;

        if (fieldName == 'area')
          stateChanges = {
            value: {
              ...this.state.value,
              area      : e.value,
              areaArabic: e.arabicName
            }
          };
        else
          stateChanges = {
            value: {
              ...this.state.value,
              [fieldName]: e.nativeEvent ? e.nativeEvent.target.value : e.value || e
            },
          };

        this.setState(stateChanges);
        this.geocodeAddress();
        this.props.onChange && this.props.onChange(stateChanges.value);
      }
    }
  }

  geocodeAddress() {
    if (!this.state.value.area)
      return;

    const geoCoder = new google.maps.Geocoder();
    const address  = this.getFormattedAddress([ 'areaArabic', 'block', 'street', 'country' ]);

    geoCoder.geocode({ address }, (results, status) => {
      if (status === google.maps.GeocoderStatus.OK) {
        this.updateLocation(results[ 0 ].geometry.location);
        this.setState({ zoom: 15 })
      }
    });
  }

  updateLocation(location) {
    var newValues = {
      ...this.state.value,
      location: {
        latitude : location.lat(),
        longitude: location.lng(),
      }
    };

    this.setState({
      value: newValues
    });

    this.props.onChange && this.props.onChange(newValues);
  }

  getFormattedAddress(fields = [ 'area', 'block', 'street', 'buildingNumber' ]) {
    const value    = this.state.value;
    const elements = [];

    if (fields.includes('area') && value.area)
      elements.push(value.area);

    if (fields.includes('areaArabic') && value.areaArabic)
      elements.push(value.areaArabic);

    if (fields.includes('block') && value.block)
      elements.push(`Block ${value.block}`);

    if (fields.includes('street') && value.street)
      elements.push(
        /\d/.test(value.street)
          ? value.street.trim().replace(/(\d)$/, '$1 street')
          : value.street
      );

    if (fields.includes('buildingNumber') && value.buildingNumber)
      elements.push(value.buildingNumber);

    if (fields.includes('country'))
      elements.push('Kuwait');

    return elements.join(', ');
  }

  handleFocus(e) {
    this.setState({ open: true });

    this.props.onFocus && this.props.onFocus(e);
  }

  getBlockOptions() {
    if (this.state.value && this.state.value.area) {
      var areaOption = areasOptions.find(it => it.value == this.state.value.area);
      return areaOption ? this.sortBlocks(areaOption.blocks) : [];
    }

    return [];
  }

  render() {
    var addressBox = this;
    var showMap    = this.props.showMap;
    var lat        = this.state.value.location.latitude || 0;
    var lng        = this.state.value.location.longitude || 0;

    return (
      <div className="AddressBox">
        <div className={this.state.open && 'invisible'}>
          <TextField
            label={this.props.label}
            value={this.getFormattedAddress()}
            onFocus={this.handleFocus}
            onChange={() => {}}
          />
        </div>
        {
          this.state.open &&
          <div ref="details" className="addressDetails">
            <div>
              <Autocomplete
                label="Area"
                options={areasOptions}
                showGroups
                {...this.getFieldProps('area')}
              />
              <div className="secondLine">
                <Autocomplete
                  label="Block"
                  showOptionsOnFocus
                  options={this.getBlockOptions()}
                  {...this.getFieldProps('block')}
                />
                <TextField
                  label="Street"
                  {...this.getFieldProps('street')}
                />
                <TextField
                  label="Jadda"
                  className="optional"
                  placeholder="Optional"
                  {...this.getFieldProps('avenue')}
                />
              </div>
              <div className="thirdLine">
                <div className="pure-u-1-3">
                  <TextField
                    label="House/Building"
                    {...this.getFieldProps('buildingNumber')}
                  />
                </div>
                <div className="pure-u-1-3">
                  <TextField
                    label="Apartment/Office"
                    className="optional"
                    placeholder="Optional"
                    {...this.getFieldProps('apartment')}
                  />
                </div>
                <div className="pure-u-1-3">
                  <TextField
                    label="Floor"
                    className="optional"
                    placeholder="Optional"
                    {...this.getFieldProps('floor')}
                  />
                </div>
              </div>

              <div className="pure-g">
                <div className="pure-u-21-24">
                  <TextField
                    label="Notes/Instructions"
                    placeholder="Optional"
                    className="optional"
                    onBlur={() => this.setState({ open: false })}
                    {...this.getFieldProps('instructions')}
                  />
                </div>
                <div className="pure-u-3-24 okContainer">
                  <Button
                    className="okButton"
                    label="OK"
                    onClick={() => this.setState({ open: false })}
                  />
                </div>
              </div>
            </div>
            {
              !!showMap &&
              <GoogleMapLoader
                containerElement={<div className="mapContainer"/>}
                googleMapElement={
                  <GoogleMap
                    zoom={this.state.zoom}
                    center={{ lat, lng }}
                    onZoomChanged={ function() { addressBox.setState({ zoom: this.zoom }) } }
                    options={{ streetViewControl: false }}
                  >
                    {
                      !!lat &&
                      <Marker
                        position={{ lat, lng }}
                        draggable={this.props.draggableMarker}
                        onDragend={({ latLng }) => this.updateLocation(latLng)}
                      />
                    }
                  </GoogleMap>
                }
              />
            }
          </div>
        }
      </div>
    );
  }
}


export default AddressBox;
