import React, { Component, PropTypes } from 'react';
import keycode        from 'keycode';
import Fuse           from 'fuse.js';
import ErrorMessage   from '../ErrorMessage';
import Label          from '../Label';

import './Autocomplete.scss';

export default class Autocomplete extends Component {
  static propTypes = {
    options           : PropTypes.array,
    showOptionsOnFocus: PropTypes.bool,
    fuzzySettings     : PropTypes.shape({
      keys      : PropTypes.array,
      threshold : PropTypes.number,
      shouldSort: PropTypes.bool
    }),
    showGroups        : PropTypes.bool,
    placeholder       : PropTypes.string,
    onChange          : PropTypes.func
  };

  constructor(props) {
    super(props);

    var value = this.getValue();

    this.state = {
      filtered   : this.props.options,
      fuzzyIndex : this.getFuzzyIndex(),
      showOptions: false,
      hoverLocked: false,
      selected   : this.props.options.find(it => it.value == value),
      active     : 0,
      query      : value
    };

    this.filterOptions    = this.filterOptions.bind(this);
    this.showOptions      = this.showOptions.bind(this);
    this.handleBlur       = this.handleBlur.bind(this);
    this.handleArrowKeys  = this.handleArrowKeys.bind(this);
    this.handleArrowClick = this.handleArrowClick.bind(this);
    this.handleFocus      = this.handleFocus.bind(this);
  }

  getLabel() {
    var value = this.getValue();

    if (!this.props.options)
      return value;

    var foundOption = this.props.options.find(it => it.value == value);
    return foundOption ? foundOption.label : value;
  }

  getValue() {
    return this.props.value
      ? this.props.value.value || this.props.value
      : '';
  }

  componentDidUpdate(prevProps) {
    if (prevProps.options != this.props.options)
      this.setState({ fuzzyIndex: this.getFuzzyIndex() });

    if (prevProps.value != this.props.value && (!prevProps.value || !this.props.value || prevProps.value.value != this.props.value.value)) {
      var value    = this.getValue();
      var label    = this.getLabel();
      var selected = this.props.options.find(it => it.value == value);

      this.setState({
        selected: selected,
        query   : label
      });

      if (selected && this.props.onChange)
        this.props.onChange(selected);
    }

    var optionList     = this.refs.optionList;
    var selectedOption = optionList && optionList.querySelector('.active');

    if (!selectedOption) return;

    var optsHeight = optionList.offsetHeight;
    var scrollTop  = optionList.scrollTop;
    var selOffset  = selectedOption.offsetTop;
    var selHeight  = selectedOption.offsetHeight;

    if (selOffset + selHeight > scrollTop + optsHeight)
      optionList.scrollTop +=
        selOffset + selHeight - scrollTop - optsHeight;

    if (selOffset <= scrollTop)
      optionList.scrollTop = selOffset;
  }

  getFuzzyIndex() {
    const options  = this.props.options || [];
    const settings = {
      keys      : [ 'label' ],
      threshold : .3,
      shouldSort: false,
      ...(this.props.fuzzySettings || {})
    };

    return new Fuse(options, settings);
  }

  filterOptions(e) {
    var keyCode = e ? e.keyCode : -1 || -1;

    var query = this.state.query;

    switch (keyCode) {
      case keycode('LEFT'):
      case keycode('RIGHT'):
        return true;

      case keycode('ESC'):
      case keycode('ENTER'):
        this.handleBlur();
        return true;

      case keycode('DOWN'):
      case keycode('UP'):
        this.showOptions();
        this.setState({
          filtered: query
            ? this.state.fuzzyIndex.search(query)
            : this.props.options
        });
        return true;

      case keycode('TAB'):
        return true;
    }

    this.setState({
      filtered   : query ? this.state.fuzzyIndex.search(query) : this.props.options,
      active     : 0,
      showOptions: true
    });

    return true;

  }

  showOptions() {
    this.setState({ showOptions: true });
  }

  handleBlur() {
    this.setState({ showOptions: false });
    this.props.onBlur && this.props.onBlur();
  }

  hoverOption(index) {
    if (!this.state.hoverLocked)
      this.setState({ active: index });
  }

  chooseCurrent() {
    if (this.state.active > this.state.filtered.length - 1)
      return;

    var selected = this.state.filtered[ this.state.active ];

    this.setState({ query: selected.label });

    if (this.props.onChange && this.state.selected != selected)
      this.props.onChange(selected);

    this.setState({ selected: selected, showOptions: false });
  }

  handleArrowKeys(e) {
    switch (e.keyCode) {
      case keycode('DOWN'):
        this.lockHover();
        e.preventDefault();
        this.setState({ active: Math.min(this.state.active + 1, this.state.filtered.length - 1) });
        return false;

      case keycode('UP'):
        this.lockHover();
        e.preventDefault();
        this.setState({ active: Math.max(0, this.state.active - 1) });
        return false;

      case keycode('ENTER'):
        if (this.state.showOptions)
          e.preventDefault();
        this.chooseCurrent();
        return true;

      case keycode('TAB'):
        if (this.state.showOptions)
          this.chooseCurrent();
        return true;
    }

    return true;
  }

  lockHover() {
    const unlockHover = () => {
      this.setState({ hoverLocked: false });
      window.removeEventListener('mousemove', unlockHover);
    };

    window.addEventListener('mousemove', unlockHover);
    this.setState({ hoverLocked: true });
  };

  renderFiltered() {
    let lastGroup = null;

    return this.state.filtered.reduce((prev, it, index) => {
      if (this.props.showGroups && it.groupId != lastGroup) {
        lastGroup = it.groupId;
        prev.push(
          <li
            key={`group_${it.groupId}`}
            className="group"
          >
            {it.groupId}
          </li>
        );
      }

      prev.push(
        <li
          key={it.value}
          className={`menuItem ${index == this.state.active && 'active'}`}
          onMouseOver={() => this.hoverOption(index)}
          onMouseDown={() => this.chooseCurrent(index)}
        >
          {it.label}
        </li>
      );
      return prev;
    }, []);
  }

  handleArrowClick() {
    if (!this.state.showOptions) {
      this.setState({ showOptions: true, filtered: this.props.options });
      this.refs.query.focus();
    }
  }

  handleFocus(e) {
    if (this.props.showOptionsOnFocus)
      this.setState({ showOptions: true, filtered: this.props.options });

    this.props.onFocus && this.props.onFocus(e);
  }

  renderOptions() {
    var shouldShowOptions = this.state.showOptions && this.state.filtered.length > 0;

    if (shouldShowOptions)
      return (
        <ul className={`menu ${this.props.showGroups && 'grouped'}`} ref="optionList">
          {this.renderFiltered()}
        </ul>
      );
  }

  render() {
    return (
      <div className="Autocomplete">
        <Label content={this.props.label}/>
        <div className="inputContainer">
          <input
            ref="query"
            onKeyDown={this.handleArrowKeys}
            onKeyUp={this.filterOptions}
            onBlur={this.handleBlur}
            onFocus={this.handleFocus}
            value={this.state.query}
            onChange={(e) => this.setState({ query: e.target.value })}
            placeholder={this.props.placeholder}
            autoComplete="off"
          />
          <i className="arrow" onClick={this.handleArrowClick}/>
          {this.renderOptions()}
        </div>
        <ErrorMessage error={this.props.error}/>
      </div>
    );
  }
}