import React, { Component, PropTypes } from 'react';
import axios from 'axios';
import Webcam         from 'webcamjs';
import Label          from '../Label';
import ErrorMessage   from '../ErrorMessage';
import dataURItoBlob  from './dataUriToBlob';
import webcamSwf      from 'file!./webcam.swf';

import './FileInput.scss';

class FileInput extends Component {
  static propTypes = {
    prefix   : PropTypes.string,
    label    : PropTypes.string,
    types    : PropTypes.string,
    camera   : PropTypes.bool,
    onChange : PropTypes.func,
    uploadUrl: PropTypes.oneOfType([
      PropTypes.func,
      PropTypes.string
    ])
  };

  constructor(props) {
    super(props);

    this.state = {
      id             : Math.random().toString(30).substr(2),
      cameraLayerOpen: false,
      frozen         : false
    };

    this.handleClickOut  = this.handleClickOut.bind(this);
    this.openCameraLayer = this.openCameraLayer.bind(this);
    this.freeze          = this.freeze.bind(this);
    this.unfreeze        = this.unfreeze.bind(this);
    this.capture         = this.capture.bind(this);
    this.handleUpload    = this.handleUpload.bind(this);
    this.handleClick     = this.handleClick.bind(this);
  }

  componentDidUpdate(prevProps, prevState) {
    if (!prevState.cameraLayerOpen && this.state.cameraLayerOpen) {
      Webcam.set({
        width       : 640,
        height      : 480,
        flip_horiz  : true,
        image_format: 'jpeg',
        jpeg_quality: 95,
        swfURL      : webcamSwf
      });

      Webcam.attach(`#camera${this.state.id}`);
    }

    if (prevState.cameraLayerOpen && !this.state.cameraLayerOpen)
      Webcam.reset();
  }


  openCameraLayer() {
    Webcam.unfreeze();

    this.setState({
      cameraLayerOpen: true,
      frozen         : false
    });
  }

  freeze() {
    Webcam.freeze();
    this.setState({ frozen: true });
  }

  unfreeze() {
    Webcam.unfreeze();
    this.setState({ frozen: false });
  }

  capture() {
    Webcam.snap((dataUri) => {
      this.handleUpload(dataUri);
      this.setState({ cameraLayerOpen: false });
    });
  }

  handleClickOut(e) {
    if (e.target == this.refs.cameraLayerContainer)
      this.setState({ cameraLayerOpen: false });
  }

  handleUpload(it) {
    const file = (typeof it == 'string')
      ? Object.assign(dataURItoBlob(it), { name: 'photo.jpg' })
      : it.target.files[ 0 ];

    if (file == null) {
      this.refs.finalUrl.value = '';
      return;
    }

    this.setState({ isLoading: true });

    new Promise(resolve => {
      resolve(
        typeof this.props.uploadUrl == 'function'
          ? this.props.uploadUrl(file.name)
          : this.props.uploadUrl
      );
    }).then(uploadUrl => {

      var data = new FormData();
      data.append('file', file);

      axios
        .put(uploadUrl, data)
        .then(res => {
          this.setState({ isLoading: false });

          var fileUrl = res.data;

          this.refs.fileUrl.value = fileUrl;
          this.props.onChange && this.props.onChange(fileUrl);
          this.props.onBlur && this.props.onBlur();

          this.setState({
            fileUrl: fileUrl,
            error  : undefined
          });

        })
        .catch(err => {
          this.setState({ isLoading: false });

          this.refs.fileInput.value = '';
          this.refs.fileUrl.value   = '';

          this.setState({
            fileUrl: '',
            error  : 'Upload failed'
          });
        });
    })
  }

  handleClick(e) {
    this.props.onFocus && this.props.onFocus(e);
  }

  render() {
    return (
      <div className={`FileInput ${this.props.className}`}>
        <Label content={this.props.label}/>
        <div className="inputContainer">
          <input
            ref="fileInput"
            type="file"
            id={this.state.id}
            accept={this.props.types}
            onChange={this.handleUpload}
            onBlur={() => console.log('blur')}
          />
          <input
            ref="fileUrl"
            type="text"
            name={this.props.name}
          />
          <label htmlFor={this.state.id}>
            {this.renderButtonContent()}
          </label>
          {this.renderCameraButton()}
          {this.renderPreviewLink()}
        </div>
        <ErrorMessage error={this.props.error || this.state.error}/>
        {this.renderCameraLayer()}
      </div>
    );
  }

  renderButtonContent() {
    const loading = this.state.isLoading;
    const fileUrl = this.state.fileUrl || this.props.value;

    if (loading)
      return (<span>Uploading...</span>);
    else
      return (
        <a disabled={loading} onClick={this.handleClick}>
          <i className="material-icons">
            {fileUrl ? 'check' : 'attach_file'}
          </i>
          {fileUrl ? 'Change file...' : 'Select file...'}
        </a>
      );
  }

  renderCameraButton() {
    if (this.props.camera && !this.state.isLoading)
      return (
        <a type="button" className="cameraButton" onClick={this.openCameraLayer}>
          <i className="material-icons">camera_alt</i>
        </a>
      );
  }

  renderPreviewLink() {
    var fileUrl = this.state.fileUrl || this.props.value;

    if (fileUrl)
      return (<div><a className="previewLink" href={fileUrl} target="_blank">Preview file</a></div>);
  }

  renderCameraLayer() {
    return (
      <div
        ref="cameraLayerContainer"
        className={`cameraLayer ${this.state.cameraLayerOpen && 'open'}`}
        onClick={this.handleClickOut}
      >
        <div className="cameraBox">
          <div className="cameraImage" id={`camera${this.state.id}`}></div>
          <div className="cameraButtons">
            {
              !this.state.frozen &&
              <i className="material-icons shootIcon" onClick={this.freeze}>camera</i>
            }
            {this.state.frozen && <i className="material-icons" onClick={this.unfreeze}>close</i>}
            {this.state.frozen && <i className="material-icons" onClick={this.capture}>check</i>}
          </div>
        </div>
      </div>
    );
  }
}

export default FileInput;
